/* dft.h - function prototypes and structures for dft and fft functions */
/**********************************************************************
DFT.C - SOURCE CODE FOR DISCRETE FOURIER TRANSFORM FUNCTIONS

dft       Discrete Fourier Transform
idft      Inverse Discrete Fourier Transform
fft       In-place radix 2 decimation in time FFT
ifft      In-place radix 2 decimation in time inverse FFT
rfft      Trig recombination real input FFT
ham       Hamming window
han       Hanning window
triang    Triangle window
black     Blackman window
harris    4 term Blackman-Harris window
log2new   Base 2 logarithm
gaussian  Generate Gaussian noise
uniform   Generate uniform noise                 
***********************************************************************/
/* COMPLEX STRUCTURE */
typedef struct {
    double real, imag;
} COMPLEX;
/* function prototypes for dft and inverse dft functions */
/***********************************************************************
void dft(COMPLEX *Datain, COMPLEX *Dataout, int N);
void idft(COMPLEX *Datain, COMPLEX *Dataout, int N);
void fft(COMPLEX *x, int m);
void ifft(COMPLEX *x, int m);
void rfft(double *x, COMPLEX *y, int m);
void ham(COMPLEX *x, int n);
void han(COMPLEX *x, int n);
void triang(COMPLEX * x,int n);
void black(COMPLEX *x, int n);
void harris(COMPLEX *x, int n);
int log2new(unsigned int x);             
double gaussian()
double uniform()
*************************************************************************/

/***********************************************************************

dft - Discrete Fourier Transform

This function performs a straight DFT of N points on an array of
complex numbers whose first member is pointed to by Datain.  The
output is placed in an array pointed to by Dataout.

void dft(COMPLEX *Datain, COMPLEX *Dataout, int N)

*************************************************************************/
void dft(COMPLEX *Datain, COMPLEX *Dataout, int N);
/***********************************************************************

idft - Inverse Discrete Fourier Transform
  
This function performs an inverse DFT of N points on an array of
complex numbers whose first member is pointed to by Datain.  The
output is placed in an array pointed to by Dataout.
It returns nothing.

void idft(COMPLEX *Datain, COMPLEX *Dataout, int N)

*************************************************************************/
void idft(COMPLEX *Datain, COMPLEX *Dataout, int N);
/**************************************************************************

fft - In-place radix 2 decimation in time FFT

Requires pointer to complex array, x and power of 2 size of FFT, m
(size of FFT = 2**m).  Places FFT output on top of input COMPLEX array.

void fft(COMPLEX *x, int m)

*************************************************************************/
void fft(COMPLEX *x, int m);
/**************************************************************************

ifft - In-place radix 2 decimation in time inverse FFT

Requires pointer to complex array, x and power of 2 size of FFT, m
(size of FFT = 2**m).  Places inverse FFT output on top of input
frequency domain COMPLEX array.

void ifft(COMPLEX *x, int m)

*************************************************************************/
void ifft(COMPLEX *x, int m);
/************************************************************

rfft - trig recombination real input FFT

Requires real array pointed to by x, pointer to complex
output array, y and the size of real FFT in power of
2 notation, m (size of input array and FFT, N = 2**m).
On completion, the COMPLEX array pointed to by y 
contains the lower N/2 + 1 elements of the spectrum.

void rfft(double *x, COMPLEX *y, int m)

***************************************************************/
void rfft(double *x, COMPLEX *y, int m);
/*************************************************************************

ham - Hamming window

Scales both real and imaginary parts of input array in-place.
Requires COMPLEX pointer and length of input array.

void ham(COMPLEX *x, int n)

*************************************************************************/
void ham(COMPLEX *x, int n);
/*************************************************************************

han - Hanning window

Scales both real and imaginary parts of input array in-place.
Requires COMPLEX pointer and length of input array.

void han(COMPLEX *x, int n)

*************************************************************************/
void han(COMPLEX *x, int n);
/*************************************************************************

triang - triangle window

Scales both real and imaginary parts of input array in-place.
Requires COMPLEX pointer and length of input array.

void triang(COMPLEX * x,int n)

*************************************************************************/
void triang(COMPLEX * x,int n);
/*************************************************************************

black - Blackman window

Scales both real and imaginary parts of input array in-place.
Requires COMPLEX pointer and length of input array.

void black(COMPLEX *x, int n)

*************************************************************************/
void black(COMPLEX *x, int n);
/*************************************************************************

harris - 4 term Blackman-Harris window

Scales both real and imaginary parts of input array in-place.
Requires COMPLEX pointer and length of input array.

void harris(COMPLEX *x, int n)

*************************************************************************/
void harris(COMPLEX *x, int n);
/**************************************************************************

log2new - base 2 logarithm

Returns base 2 log such that i = 2**ans where ans = log2new(i).
if log2new(i) is between two values, the larger is returned.

int log2new(unsigned int x)

*************************************************************************/
int log2new(unsigned int x);
/**************************************************************************

gaussian - generates zero mean unit variance Gaussian random numbers

Returns one zero mean unit variance Gaussian random numbers as a double.
Uses the Box-Muller transformation of two uniform random numbers to
Gaussian random numbers.

double gaussian()

*************************************************************************/
double gaussian();
/**************************************************************************

uniform - generates zero mean uniform random number from -0.5 to 0.5

Returns one zero mean uniformly distributed random number as a double.
No arguments, calls function rand() from the C library.

double uniform()

*************************************************************************/
double uniform();

